library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity signal_generator is
    port (
        clk        : in  std_logic;
        reset      : in  std_logic; 
        count      : integer range 0 to MAX_COUNT; 
        sw         : in std_logic;  
        val        : out std_logic_vector(RAMP_BITS-1 downto 0); -- sync signal (ramp)
        counter    : out integer range 0 to MAX_COUNT;      
        init       : out std_logic;    -- initial ramp point (0)
        mid        : out std_logic;    -- middle  ramp point (255)
        tick       : out std_logic;
        enable     : out std_logic     -- if the signal generator is working
    );
end signal_generator;

architecture Behavioral of signal_generator is
    signal tick_aux     : std_logic;
    
begin
    tick    <= tick_aux;
    enable  <= not sw;
    counter <= count; 
                    
    
                     
    freq_gen_unit: entity work.freq_gen(behavioral)
        port map (
            clk=>clk, reset=>reset, count=>count, sw=>sw, tick=>tick_aux
        ); 
                     
    ramp_gen_unit: entity work.ramp_gen(behavioral)
        port map (
            clk=>clk, reset=>reset, tick=>tick_aux, 
            sw=>sw, val=>val, init=>init, mid=>mid
        );
end Behavioral;
