library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity uart is
    generic (
        B       : natural := 8;    -- number of data bits
        W       : natural := 8;    -- Number of adress bits
        s_ticks : natural := 16; 
        s_bits  : natural := 16;   -- number of stop bits
        dvsr    : natural := DVSR; -- see config.vhd
        dvsr_bit: natural := DVSR_BIT
    );
    port (
        clk         : in  std_logic;
        reset       : in  std_logic;
        wr_uart     : in  std_logic; 
        wr_data     : in  std_logic_vector(B-1 downto 0);
        rx          : in  std_logic; 
        clr_flag    : in  std_logic; 
        flag        : out std_logic; 
        tx_full     : out std_logic;
        tx_empty    : out std_logic;
        tx          : out std_logic
    );
end uart;

architecture behavioral of uart is
    signal baud_tick    : std_logic;
    signal tx_start     : std_logic; 
    signal dinp         : std_logic_vector(B-1 downto 0); 
    signal tx_done_tick : std_logic;
    signal rx_done_tick : std_logic; 
    signal dout_rx      : std_logic_vector(B-1 downto 0); 
    signal tx_empty_aux : std_logic;
begin
    tx_start <= not tx_empty_aux;
    tx_empty <= tx_empty_aux;
    
    freq_gen_unit: entity work.freq_gen(behavioral)
        port map(clk => clk, reset => reset, count => dvsr,
            sw => '0', tick => baud_tick); 
            
    uart_rx_unit: entity work.uart_rx(behavioral)
        generic map(d_bits => B, s_ticks => s_ticks)
        port map(clk => clk, reset => reset, rx => rx, baud_tick => baud_tick,
                done_tick => rx_done_tick, dout => dout_rx); 
                
    flag_rx_unit: entity work.flag_rx(behavioral)
        generic map(B => B)
        port map(clk => clk, reset => reset, set_flag => rx_done_tick, 
                clr_flag => clr_flag, dout_rx => dout_rx, flag => flag);             
       
    uart_tx_unit: entity work.uart_tx(behavioral)
        generic map(B => B, s_ticks => s_ticks)
        port map(clk => clk, reset => reset, baud_tick => baud_tick, tx_start => tx_start, 
            dinp => dinp, tx => tx, done_tick => tx_done_tick ); 
    
    fifo_tx_unit: entity work.fifo(behavioral)
        generic map(B => B, W => W)
        port map(clk => clk, reset => reset, rd => tx_done_tick, 
            wr => wr_uart, wr_data => wr_data, rd_data => dinp, full => tx_full, 
            empty => tx_empty_aux);
            
end behavioral;

